@@ -1,20 +1,24 @@
 from __future__ import annotations
 from typing import Any, NamedTuple, TypeVar, Union, cast, TYPE_CHECKING
-from jsonclasses import (get_fields, types, Field, FieldType, FieldStorage,
-                         resolve_types, ObjectGraph, concat_keypath)
 from datetime import datetime
 from inflection import camelize
 from bson.objectid import ObjectId
+from jsonclasses.jsonclass_field import JSONClassField
+from jsonclasses.field_definition import FieldStorage, FieldType
+from jsonclasses.keypath_utils import concat_keypath
+from jsonclasses.types_resolver import TypesResolver
+from jsonclasses.mark_graph import MarkGraph
+from jsonclasses.types import types
 from .coder import Coder
 from .utils import ref_db_field_key, ref_db_field_keys
 from .context import EncodingContext
 from .command import (Command, InsertOneCommand, UpdateOneCommand,
                       UpsertOneCommand, BatchCommand)
-from .connector import connector
+from .connection import Connection
 
 if TYPE_CHECKING:
-    from .base_mongo_object import BaseMongoObject
-    T = TypeVar('T', bound=BaseMongoObject)
+    from .pymongo_object import PymongoObject
+    T = TypeVar('T', bound=PymongoObject)
 
 
 class EncodingResult(NamedTuple):
@@ -30,9 +34,9 @@ class Encoder(Coder):
         if context.value is None:
             return EncodingResult(result=None, commands=[])
         value = cast(list[Any], context.value)
-        fd = context.types.fdesc
-        item_types = resolve_types(fd.raw_item_types,
-                                   graph_sibling=context.root.__class__)
+        fd = context.types.definition
+        item_types = TypesResolver().resolve_types(
+            fd.raw_item_types, context.root.__class__.definition.config)
         if fd.field_storage == FieldStorage.FOREIGN_KEY:
             item_types = item_types.linkedby(cast(str, fd.foreign_key))
         if fd.field_storage == FieldStorage.LOCAL_KEY:
@@ -55,9 +59,9 @@ class Encoder(Coder):
         if context.value is None:
             return EncodingResult(result=None, commands=[])
         value = cast(dict[str, Any], context.value)
-        fd = context.types.fdesc
-        item_types = resolve_types(fd.raw_item_types)
-        camelized = context.owner.__class__.config.camelize_db_keys
+        fd = context.types.definition
+        item_types = TypesResolver().resolve_types(fd.raw_item_types)
+        camelized = context.owner.__class__.dbconf.camelize_db_keys
         result = {}
         commands = []
         for key, item in value.items():
@@ -76,13 +80,13 @@ class Encoder(Coder):
         if context.value is None:
             return EncodingResult(result=None, commands=[])
         value = cast(dict[str, Any], context.value)
-        fd = context.types.fdesc
+        fd = context.types.definition
         shape_types = cast(dict[str, Any], fd.shape_types)
-        camelized = context.owner.__class__.config.camelize_db_keys
+        camelized = context.owner.__class__.dbconf.camelize_db_keys
         result = {}
         commands = []
         for key, item in value.items():
-            item_types = resolve_types(shape_types[key])
+            item_types = TypesResolver().resolve_types(shape_types[key])
             item_result, item_commands = self.encode_item(context.new(
                 value=item,
                 types=item_types,
@@ -95,21 +99,22 @@ class Encoder(Coder):
         return EncodingResult(result, commands)
 
     def _join_command(self,
-                      this_instance: BaseMongoObject,
-                      this_field: Field,
-                      that_cls: type[BaseMongoObject],
+                      this_instance: PymongoObject,
+                      this_field: JSONClassField,
+                      that_cls: type[PymongoObject],
                       that_id: ObjectId) -> UpsertOneCommand:
         this_cls = this_instance.__class__
+        connection = Connection.from_class(this_instance.__class__)
         this_cls_name = this_cls.__name__
         that_cls_name = that_cls.__name__
         this_field_name = ref_db_field_key(this_cls_name, this_cls)
         that_field_name = ref_db_field_key(that_cls_name, that_cls)
         join_table_name = self.join_table_name(
             this_cls,
-            this_field.field_name,
+            this_field.name,
             that_cls,
-            cast(str, this_field.fdesc.foreign_key))
-        collection = connector._database.get_collection(join_table_name)
+            cast(str, this_field.definition.foreign_key))
+        collection = connection.collection(join_table_name)
         this_id = ObjectId(this_instance._id)
         matcher = {
             this_field_name: this_id,
@@ -122,33 +127,33 @@ class Encoder(Coder):
     def encode_instance(self,
                         context: EncodingContext,
                         root: bool = False) -> EncodingResult:
-        from .base_mongo_object import BaseMongoObject
+        from .pymongo_object import PymongoObject
         if context.value is None:
             return EncodingResult(result=None, commands=[])
-        value = cast(BaseMongoObject, context.value)
+        value = cast(PymongoObject, context.value)
         cls = value.__class__
         id = cast(Union[str, int], value._id)
-        if context.object_graph.getp(cls, id) is not None:
+        if context.mark_graph.getp(cls, id) is not None:
             return EncodingResult({'_id': ObjectId(id)}, commands=[])
-        context.object_graph.put(value)
-        instance_fd = context.types.fdesc
+        context.mark_graph.put(value)
+        instance_fd = context.types.definition
         write_instance = instance_fd.field_storage != FieldStorage.EMBEDDED
         if root:
             write_instance = True
         use_insert_command = False
-        fields_need_update: set[str] = value.modified_fields_p
+        fields_need_update: set[str] = value.persisted_modified_fields
         if value.is_new:
             use_insert_command = True
         result_set = {}
         matcher = {}
         commands = []
         result_addtoset = {}
-        for field in get_fields(value):
-            if field.fdesc.is_temp_field:
+        for field in value.__class__.definition.fields:
+            if field.definition.is_temp_field:
                 continue
-            fname = field.field_name
+            fname = field.name
             fvalue = getattr(value, fname)
-            ftypes = field.field_types
+            ftypes = field.types
             if self.is_id_field(field):
                 if use_insert_command:
                     result_set['_id'] = ObjectId(fvalue)
@@ -235,10 +240,13 @@ class Encoder(Coder):
                     keypath_parent=fname,
                     parent=value))
                 if use_insert_command or fname in fields_need_update:
-                    result_set[field.db_field_name] = item_result
+                    if context.owner.__class__.dbconf.camelize_db_keys:
+                        result_set[camelize(field.name, False)] = item_result
+                    else:
+                        result_set[field.name] = item_result
                 commands.extend(item_commands)
         if write_instance:
-            collection = value.__class__.collection()
+            collection = Connection.get_collection(value.__class__)
             if use_insert_command:
                 insert_command = InsertOneCommand(collection, result_set)
                 commands.append(insert_command)
@@ -255,12 +263,13 @@ class Encoder(Coder):
         setattr(value, '_is_new', False)
         setattr(value, '_is_modified', False)
         setattr(value, '_modified_fields', set())
+        setattr(value, '_previous_values', {})
         return EncodingResult(result_set, commands)
 
     def encode_item(self, context: EncodingContext) -> EncodingResult:
         if context.value is None:
             return EncodingResult(result=None, commands=[])
-        field_type = context.types.fdesc.field_type
+        field_type = context.types.definition.field_type
         if field_type == FieldType.LIST:
             return self.encode_list(context)
         elif field_type == FieldType.DICT:
@@ -273,6 +282,8 @@ class Encoder(Coder):
             return EncodingResult(
                 result=datetime.fromisoformat(context.value.isoformat()),
                 commands=[])
+        elif field_type == FieldType.ENUM:
+            return EncodingResult(result=context.value.value, commands=[])
         else:
             return EncodingResult(context.value, [])
 
@@ -286,5 +297,5 @@ class Encoder(Coder):
             owner=root,
             keypath_parent='',
             parent=root,
-            object_graph=ObjectGraph()), root=True)[1]
+            mark_graph=MarkGraph()), root=True)[1]
         return BatchCommand(commands=commands)
